unit FrameEditUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, SynEditHighlighter, SynHighlighterAny, SynEditPrint,
  SynEditOptionsDialog, SynEditMiscClasses, SynEditSearch,
  SynCompletionProposal, SynEdit, ELSuperEdit, Menus, SynExportRTF,
  SynEditExport, SynExportHTML;

type
  TFrameEdit = class(TFrame)
    Completion: TSynCompletionProposal;
    EditSearch: TSynEditSearch;
    EditOptionsDialog: TSynEditOptionsDialog;
    EditPrint: TSynEditPrint;
    FreeBasic: TSynAnySyn;
    Edit: TELSuperEdit;
    PopupMenu: TPopupMenu;
    menuUndo: TMenuItem;
    menuRedo: TMenuItem;
    menuPaste: TMenuItem;
    menuCut: TMenuItem;
    menuCopy: TMenuItem;
    menuDelete: TMenuItem;
    N1: TMenuItem;
    menuSelectAll: TMenuItem;
    menuComment: TMenuItem;
    menuOn: TMenuItem;
    menuOff: TMenuItem;
    menuNormalize: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    N4: TMenuItem;
    menuProperties: TMenuItem;
    ExporterHTML: TSynExporterHTML;
    ExporterRTF: TSynExporterRTF;
    procedure EditChange(Sender: TObject);
    procedure EditStatusChange(Sender: TObject;
      Changes: TSynStatusChanges);
    procedure EditGutterClick(Sender: TObject; Button: TMouseButton; X, Y,
      Line: Integer; Mark: TSynEditMark);
    procedure menuUndoClick(Sender: TObject);
    procedure menuRedoClick(Sender: TObject);
    procedure menuPasteClick(Sender: TObject);
    procedure menuCutClick(Sender: TObject);
    procedure menuCopyClick(Sender: TObject);
    procedure menuDeleteClick(Sender: TObject);
    procedure menuSelectAllClick(Sender: TObject);
    procedure menuNormalizeClick(Sender: TObject);
    procedure menuOnClick(Sender: TObject);
    procedure menuOffClick(Sender: TObject);
    procedure menuPropertiesClick(Sender: TObject);
    procedure PopupMenuPopup(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
   oc:TSynEditorOptionsContainer;

implementation

{$R *.dfm}

uses MainUnit, PageSheet, SettingsUnit;

procedure TFrameEdit.EditChange(Sender: TObject);
begin   
    TPageSheet(Parent).Saved:=false;
    Debugger.Reset;
end;

procedure TFrameEdit.EditStatusChange(Sender: TObject;  Changes: TSynStatusChanges);
begin
    if (scCaretX in Changes) or (scCaretY in Changes) then
       Main.StatusBar.Panels[0].Text:=format('Line: %d, Col: %d',[TSynEdit(Sender).CaretY,TSynEdit(Sender).CaretX]);
    if (scLeftChar in Changes) then ;
    if (scTopLine in Changes) then ;
    if (scInsertMode in Changes) then Main.StatusBar.Panels[1].Text:='Insert';
    if (scModified in Changes) then Main.StatusBar.Panels[1].Text:='Modified';
end;

procedure TFrameEdit.EditGutterClick(Sender: TObject; Button: TMouseButton;
  X, Y, Line: Integer; Mark: TSynEditMark);
begin
    TSynEdit(Sender).Marks.Clear;
end;

procedure TFrameEdit.menuUndoClick(Sender: TObject);
begin
    Edit.Undo
end;

procedure TFrameEdit.menuRedoClick(Sender: TObject);
begin
    Edit.Redo
end;

procedure TFrameEdit.menuPasteClick(Sender: TObject);
begin
    Edit.PasteFromClipboard
end;

procedure TFrameEdit.menuCutClick(Sender: TObject);
begin
    Edit.CutToClipboard
end;

procedure TFrameEdit.menuCopyClick(Sender: TObject);
begin
    Edit.CopyToClipboard
end;

procedure TFrameEdit.menuDeleteClick(Sender: TObject);
begin
    Edit.ClearSelection
end;

procedure TFrameEdit.menuSelectAllClick(Sender: TObject);
begin
   Edit.SelectAll
end;

procedure TFrameEdit.menuNormalizeClick(Sender: TObject);
begin
    if pos(#9,Edit.SelText)>0 then
       Edit.SelText:=StringReplace(Edit.SelText,#9,' ',[rfreplaceall]);
end;

procedure TFrameEdit.menuOnClick(Sender: TObject);
begin
    if Edit.SelText<>'' then
       Edit.SelText:='/'''+Edit.SelText+'''/'
end;

procedure TFrameEdit.menuOffClick(Sender: TObject);
var
   s:string;
begin
    s:=Edit.SelText;
    if s<>'' then begin
       s:=StringReplace(s,'''/','',[rfreplaceall]);
       s:=StringReplace(s,'/''','',[rfreplaceall]);
       s:=StringReplace(s,'''','',[rfreplaceall]);
    end  ;
    Edit.SelText:=s;
end;

procedure TFrameEdit.menuPropertiesClick(Sender: TObject);
begin

    //oc.Assign(Edit);
    if EditOptionsDialog.Execute(oc) then
       oc.AssignTo(Edit);
    Launcher.LineNumbers:=oc.Gutter.ShowLineNumbers;
    Launcher.FontName:=oc.Font.Name;
    Launcher.FontSize:=oc.Font.Size;
end;

procedure TFrameEdit.PopupMenuPopup(Sender: TObject);
begin
    menuRedo.Enabled:=Edit.CanRedo;
    menuUndo.Enabled:=Edit.CanUndo;
    menuPaste.Enabled:=Edit.CanPaste;
    menuCopy.Enabled:=Edit.SelText<>'';
    menuCut.Enabled:=Edit.SelText<>'';
    menuDelete.Enabled:=Edit.SelText<>'';
    menuSelectAll.Enabled:=Edit.Text<>'';
    menuNormalize.Enabled:=(Edit.SelText<>'') and (pos(#9,Edit.SelText)>0);
    menuOn.Enabled:=(Edit.SelText<>'') and ((pos(#39,Edit.SelText)=0) and (pos('/''',Edit.SelText)=0));
    menuOff.Enabled:=(Edit.SelText<>'') and ((pos(#39,Edit.SelText)>0) or (pos('/''',Edit.SelText)>0));

end;

procedure TFrameEdit.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    if key=vk_return then
       TPageSheet(Parent).scanner.Scan
end;

initialization
    RegisterClasses([TELSuperEdit]);
    oc:=TSynEditorOptionsContainer.Create(nil);
finalization
    unRegisterClasses([TELSuperEdit]);
    oc.Free;

end.
