unit PageSheet;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, ComCtrls, FrameEditUnit, ScannerUnit;

type
   TPageSheet=class(TTabSheet)
   private
     fFrame:TFrameEdit;
     fFileName:string;
     fSaved:boolean;
     fScanner:TScanner;
     fTree:TTreeView;
     procedure SetSaved(v:boolean);
     procedure SetFileName(v:string);
     procedure SetTree(v:TTreeView);
   public
     constructor Create(AOwner:TComponent);override;
     destructor Destroy;override;
     procedure Save;
     procedure SaveAs;
     procedure Dispose;
     property Frame:TFrameEdit read fFrame;
     property FileName:string read fFileName write SetFileName;
     property Saved:boolean read fSaved write SetSaved;
     property Scanner:TScanner read fScanner;
     property Tree:TTreeView read fTree write SetTree;
   end;

var
   PageID:integer;

implementation

uses MainUnit;

constructor TPageSheet.Create(AOwner:TComponent);
begin
   inherited;
   fFrame:=TFrameEdit.Create(self);
   fFrame.Align:=alClient;
   fFrame.Parent:=self;
   ActivePage:=Self;
   fSaved:=true;
   fScanner:=TScanner.create;
   fScanner.Edit:=fFrame.Edit;
end;

destructor TPageSheet.Destroy;
begin
   fScanner.Free;
   inherited
end;

procedure TPageSheet.SetSaved(v:boolean);
begin
    fSaved:=v;
    if v then
       if pos('*',Caption)>0 then Caption:=StringReplace(Caption,'*','',[]);
    if not v then
       if pos('*',Caption)=0 then Caption:='*'+Caption;
end;

procedure TPageSheet.SetFileName(v:string);
begin
   fFileName:=v;
   if FileExists(v) then begin
      fFrame.Edit.Lines.LoadFromFile(v);
      Caption:=ExtractFileName(v);
      Saved:=true ;
      fScanner.FileName:=v;
   end else begin
      inc(PageID);
      Caption:=format('noname%d',[PageID]);
      fFrame.Edit.Lines.Text:=v;
   end ;
end;

procedure TPageSheet.SetTree(v:TTreeView);
begin
    fTree:=v;
    if v<>nil then begin
       fScanner.Tree:=v;
       fScanner.Scan;
    end
end;

procedure TPageSheet.Save;
begin
   if FileExists(fFileName) then begin
       fFrame.Edit.Lines.SaveToFile(fFileName);
       Saved:=true
   end else SaveAs;
end;

procedure TPageSheet.SaveAs;
begin
     with TSaveDialog.Create(nil) do begin
            Filter:='FreeBasic file (*.bas)|*.bas|FreeBasic include (*.bi)|*.bi|All files (*.*)|*.*';
            FileName:=fFileName;
            if FileName='' then
               FileName:=StringReplace(Caption,'*','',[]);
            if Execute then begin
               case FilterIndex of
               1:FileName:=ChangeFileExt(FileName,'.bas');
               2:FileName:=ChangeFileExt(FileName,'.bi');
               end;
               fFrame.Edit.Lines.SaveToFile(FileName);
               Caption:=ExtractFileName(FileName);
               fFileName:=FileName;
               Saved:=true;
            end;
            Free;
    end;
end;

procedure TPageSheet.Dispose;
var
   s:string;
begin
    s:=fFileName;
    if s='' then
       s:=Caption;
    if not Saved then
       case messageDlg(format('The ''%s'' file was modified.'#10'Do you want to save?',[s]),mtConfirmation,[mbyes,mbno],0) of
            mryes: if FileExists(fFileName) then Save else SaveAs;
       end;
    Free
end;

end.
