unit SettingsUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, IniFiles, TypesUnit, CompilerUnit;

type
  TLauncher=class
    Minimize,Disable,Plugins,Completion,
    LineNumbers,KeepSwitch,IntegrateIn: boolean;
    Icon,Help,FontName,Language:string;
    FontSize:integer;
    SelClass:string;
    SelType:TType;
    Lib:TLibrary;
    Types,Classes,RegisterClasses:TStrings;
    ClassPalettes:TClassRegPages;
    function TypeExists(v:string):TType;
    procedure Load;
    procedure Save;
    constructor Create;
    destructor Destroy; override;
  end;

  TSettings = class(TForm)
    cbxMinimize: TCheckBox;
    cbxDisable: TCheckBox;
    cbxPlugins: TCheckBox;
    cbxCompletion: TCheckBox;
    ImageIcon: TImage;
    btnIcon: TBitBtn;
    LabelHelp: TLabel;
    edHelp: TEdit;
    btnHelp: TBitBtn;
    Bevel: TBevel;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    procedure btnIconClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Settings: TSettings;
  iniFile:string;
  ResultVisible:boolean;

implementation

{$R *.dfm}

uses MainUnit, LanguagesUnit, ResultUnit;

{ TLauncher }
constructor TLauncher.Create;
begin
    Types:=TStringList.Create;
    Classes:=TStringList.Create;
    RegisterClasses:=TStringList.Create;
    Lib:=TLibrary.create;
    KeepSwitch:=true;
end;

destructor TLauncher.Destroy;
begin
    Types.Free;
    Classes.Free;
    RegisterClasses.Free;
    Lib.Free;
    inherited
end;

function TLauncher.TypeExists(v:string):TType;
var
   i:integer;
begin
   result:=nil;
   if v='' then exit;
   for i:=0 to Types.Count-1 do
       if CompareText(TType(Types.Objects[i]).Hosted,v)=0 then begin
          result:=TType(Types.Objects[i]);
          break
       end
end;

procedure TLauncher.Load;
begin
    with TIniFile.Create(iniFile) do begin
         Main.ToolBar.Visible:=ReadBool('Launcher','ToolBar',true);
         Main.StatusBar.Visible:=ReadBool('Launcher','StatusBar',true);
         IntegrateIn:=ReadBool('Launcher','IntegrateIn',false);
         ResultVisible:=ReadBool('Launcher','Results',true);
         KeepSwitch:=ReadBool('Launcher','KeepSwitch',false);
         Minimize:=ReadBool('Launcher','Minimize',true);
         Disable:=ReadBool('Launcher','Disable',false);
         Plugins:=ReadBool('Launcher','Plugins',false);
         Completion:=ReadBool('Launcher','Completion',false);
         LineNumbers:=ReadBool('Launcher','LineNumbers',true);
         FontName:=ReadString('Launcher','FontName','Tahoma');
         FontSize:=ReadInteger('Launcher','FontSize',9);
         Compiler.Switch:=ReadString('Launcher','Switch','');
         Help:=ReadString('Launcher','Help','');
         Application.HelpFile:=Help;
         Icon:=ReadString('Launcher','Icon','');
         if Main<>nil then begin
            Main.Width:=ReadInteger('Main','Width',700);
            Main.Height:=ReadInteger('Main','Height',500);
            Main.Left:=ReadInteger('Main','Left',(screen.WorkAreaWidth-Main.width) div 2);
            Main.Top:=ReadInteger('Main','Top',(screen.WorkAreaHeight-Main.Height) div 2);
         end;
         if Results<>nil then begin
            Results.Width:=ReadInteger('Results','Width',700);
            Results.Height:=ReadInteger('Results','Height',500);
            Results.Left:=ReadInteger('Results','Left',(screen.WorkAreaWidth-Results.width) div 2);
            Results.Top:=ReadInteger('Results','Top',(screen.WorkAreaHeight-Results.Height) div 2);
         end;
         self.Language:=ReadString('Main','ActiveLang',wDir+'Languages\base.txt');
         if FileExists(self.Language) then begin
            ActiveLang:=TLang.Create;
            ActiveLang.Name:=ChangeFileExt(ExtractFileName(Launcher.Language),'');
            ActiveLang.FileName:=Launcher.Language;
            ActiveLang.SetLanguage;
         end ;
         Free;
    end;
    if Help='' then Application.HelpFile:='Helps\FreeBasic.chm';
end;

procedure TLauncher.Save;
begin
    with TIniFile.Create(iniFile) do begin
         WriteBool('Launcher','ToolBar',Main.ToolBar.Visible);
         WriteBool('Launcher','StatusBar',Main.StatusBar.Visible);
         WriteBool('Launcher','Results',Results.Visible);
         WriteString('Launcher','Help',Application.HelpFile);
         WriteString('Launcher','Switch',Compiler.Switch);
         WriteBool('Launcher','IntegrateIn',IntegrateIn);
         WriteBool('Launcher','KeepSwitch',KeepSwitch);
         WriteBool('Launcher','Minimize',Minimize);
         WriteBool('Launcher','Disable',Disable);
         WriteBool('Launcher','Plugins',Plugins);
         WriteBool('Launcher','Completion',Completion);
         WriteString('Launcher','Help',Help);
         WriteString('Launcher','Icon',Icon);
         WriteBool('Launcher','LineNumbers',LineNumbers);
         WriteString('Launcher','FontName',FontName);
         WriteInteger('Launcher','FontSize',FontSize);
         if Main<>nil then begin
            WriteInteger('Main','Left',Main.Left);
            WriteInteger('Main','Top',Main.Top);
            WriteInteger('Main','Width',Main.Width);
            WriteInteger('Main','Height',Main.Height);
         end;
         if Results<>nil then begin
            WriteInteger('Results','Left',Results.Left);
            WriteInteger('Results','Top',Results.Top);
            WriteInteger('Results','Width',Results.Width);
            WriteInteger('Results','Height',Results.Height);
         end;
         Free;
    end; 
end;

procedure TSettings.btnIconClick(Sender: TObject);
begin
    with TOpenDialog.Create(nil) do begin
         Filter:='Icon file (*.ico)|*.ico';
         if Execute then begin
            ImageIcon.Hint:=FileName;
            ImageIcon.Picture.LoadFromFile(FileName);
         end;
         Free
    end;
end;

procedure TSettings.btnHelpClick(Sender: TObject);
begin
    with TOpenDialog.Create(nil) do begin
         Filter:='Compiled HTML file (*.chm)|*.chm|All files (*.*)|*.*';
         if Execute then begin
            edHelp.Text:=FileName;
         end;
         Free
    end;
end;

procedure TSettings.FormShow(Sender: TObject);
begin
    cbxMinimize.Checked:=Launcher.Minimize;
    cbxDisable.Checked:=Launcher.Disable;
    cbxPlugins.Checked:=Launcher.Plugins;
    cbxCompletion.Checked:=Launcher.Completion;
    edHelp.Text:=Launcher.Help;
    ImageIcon.Hint:=Launcher.Icon;
    if FileExists(Launcher.Icon) then
       ImageIcon.Picture.LoadFromFile(Launcher.Icon);
end;

procedure TSettings.btnOKClick(Sender: TObject);
begin
    Launcher.Minimize:=cbxMinimize.Checked;
    Launcher.Disable:=cbxDisable.Checked;
    Launcher.Plugins:=cbxPlugins.Checked;
    Launcher.Completion:=cbxCompletion.Checked;
    Launcher.Help:=edHelp.Text;
    Launcher.Icon:=ImageIcon.Hint;
end;

initialization
   iniFile:=ChangeFileExt(ParamStr(0),'.ini');
finalization
end.
